@@ -90,6 +90,10 @@ public:
     FFmpegInput();
     ~FFmpegInput() override;
     const char* format_name(void) const override { return "FFmpeg movie"; }
+    int supports(string_view feature) const override
+    {
+        return (feature == "multiimage");
+    }
     bool valid_file(const std::string& name) const override;
     bool open(const std::string& name, ImageSpec& spec) override;
     bool close(void) override;
@@ -126,6 +130,7 @@ private:
     std::vector<uint8_t> m_rgb_buffer;
     std::vector<int> m_video_indexes;
     int m_video_stream;
+    int m_data_stream;
     int64_t m_frames;
     int m_last_search_pos;
     int m_last_decoded_pos;
@@ -148,6 +153,7 @@ private:
         m_rgb_buffer.clear();
         m_video_indexes.clear();
         m_video_stream     = -1;
+        m_data_stream      = -1;
         m_frames           = 0;
         m_last_search_pos  = 0;
         m_last_decoded_pos = 0;
@@ -259,6 +265,14 @@ FFmpegInput::open(const std::string& name, ImageSpec& spec)
         errorfmt("\"{}\" could not find a valid videostream", file_name);
         return false;
     }
+    for (unsigned int i = 0; i < m_format_context->nb_streams; i++) {
+        if (stream_codec(i)->codec_type == AVMEDIA_TYPE_DATA) {
+            if (m_data_stream < 0) {
+                m_data_stream = i;
+                break;
+            }
+        }
+    }
 
     // codec context for videostream
 #if USE_FFMPEG_3_1
@@ -305,9 +319,7 @@ FFmpegInput::open(const std::string& name, ImageSpec& spec)
                                & AV_CODEC_CAP_DELAY);
 
     AVStream* stream = m_format_context->streams[m_video_stream];
-    if (stream->avg_frame_rate.num != 0 && stream->avg_frame_rate.den != 0) {
-        m_frame_rate = stream->avg_frame_rate;
-    }
+    m_frame_rate     = av_guess_frame_rate(m_format_context, stream, NULL);
 
     m_frames     = stream->nb_frames;
     m_start_time = stream->start_time;
@@ -491,7 +503,7 @@ FFmpegInput::open(const std::string& name, ImageSpec& spec)
     }
 
     m_spec   = ImageSpec(m_codec_context->width, m_codec_context->height,
-                       nchannels, datatype);
+                         nchannels, datatype);
     m_stride = (size_t)(m_spec.scanline_bytes());
 
     m_rgb_buffer.resize(av_image_get_buffer_size(m_dst_pix_format,
@@ -510,6 +522,26 @@ FFmpegInput::open(const std::string& name, ImageSpec& spec)
                               AV_DICT_IGNORE_SUFFIX))) {
         m_spec.attribute(tag->key, tag->value);
     }
+    tag = NULL;
+    if (m_data_stream >= 0) {
+        while ((
+            tag = av_dict_get(m_format_context->streams[m_data_stream]->metadata,
+                              "", tag, AV_DICT_IGNORE_SUFFIX))) {
+            if (strcmp(tag->key, "timecode") == 0) {
+                m_spec.attribute("ffmpeg:TimeCode", tag->value);
+                break;
+            }
+        }
+    }
+    tag = NULL;
+    while (
+        (tag = av_dict_get(m_format_context->streams[m_video_stream]->metadata,
+                           "", tag, AV_DICT_IGNORE_SUFFIX))) {
+        if (strcmp(tag->key, "timecode") == 0) {
+            m_spec.attribute("ffmpeg:TimeCode", tag->value);
+            break;
+        }
+    }
     int rat[2] = { m_frame_rate.num, m_frame_rate.den };
     m_spec.attribute("FramesPerSecond", TypeRational, &rat);
     m_spec.attribute("oiio:Movie", true);
