@@ -30,7 +30,8 @@ public:
     // in bytes, between subsequent color channels, pixels, and scanlines.
     virtual void apply(float* data, int width, int height, int channels,
                        stride_t chanstride, stride_t xstride,
-                       stride_t ystride) const = 0;
+                       stride_t ystride) const
+        = 0;
     // Convert a single 3-color
     void apply(float* data)
     {
@@ -94,7 +95,7 @@ public:
     /// This routine will return the error string (and by default, clear any
     /// error flags).  If no error has occurred since the last time
     /// geterror() was called, it will return an empty string.
-    std::string geterror(bool clear = true);
+    std::string geterror(bool clear = true) const;
 
     /// Get the number of ColorSpace(s) defined in this configuration
     int getNumColorSpaces() const;
@@ -102,6 +103,12 @@ public:
     /// Query the name of the specified ColorSpace.
     const char* getColorSpaceNameByIndex(int index) const;
 
+    /// Given a color space name, return the index of an equivalent color
+    /// space, or -1 if not found. It will first look for an exact match of
+    /// the name, but if not found, will match a color space that is
+    /// "equivalent" to the named color space.
+    int getColorSpaceIndex(string_view name) const;
+
     /// Get the name of the color space representing the named role,
     /// or NULL if none could be identified.
     const char* getColorSpaceNameByRole(string_view role) const;
@@ -168,7 +175,7 @@ public:
     /// inverse==true, request the inverse transformation.  The
     /// context_key and context_value can optionally be used to establish
     /// extra key/value pairs in the OCIO context if they are comma-
-    /// separated lists of ontext keys and values, respectively.
+    /// separated lists of context keys and values, respectively.
     ///
     /// The handle is actually a shared_ptr, so when you're done with a
     /// ColorProcess, just discard it. ColorProcessor(s) remain valid even
@@ -252,14 +259,26 @@ public:
     ColorProcessorHandle
     createDisplayTransform(string_view display, string_view view,
                            string_view inputColorSpace, string_view looks = "",
-                           string_view context_key   = "",
+                           bool inverse = false, string_view context_key = "",
                            string_view context_value = "") const;
     ColorProcessorHandle
     createDisplayTransform(ustring display, ustring view,
                            ustring inputColorSpace, ustring looks = ustring(),
+                           bool inverse          = false,
                            ustring context_key   = ustring(),
                            ustring context_value = ustring()) const;
 
+    // OIIO_DEPRECATED("prefer the kind that takes an `inverse` parameter (2.5)")
+    ColorProcessorHandle
+    createDisplayTransform(string_view display, string_view view,
+                           string_view inputColorSpace, string_view looks,
+                           string_view context_key,
+                           string_view context_value = "") const;
+    // OIIO_DEPRECATED("prefer the kind that takes an `inverse` parameter (2.5)")
+    ColorProcessorHandle createDisplayTransform(
+        ustring display, ustring view, ustring inputColorSpace, ustring looks,
+        ustring context_key, ustring context_value = ustring()) const;
+
     /// Construct a processor to perform color transforms determined by an
     /// OpenColorIO FileTransform. It is possible that this will return an
     /// empty handle if the FileTransform doesn't exist or is not allowed.
@@ -297,6 +316,15 @@ public:
     /// is found.
     string_view parseColorSpaceFromString(string_view str) const;
 
+    /// Turn the name, which could be a color space, an alias, a role, or
+    /// an OIIO-understood universal name (like "sRGB") into a canonical
+    /// color space name. If the name is not recognized, return "".
+    string_view resolve(string_view name) const;
+
+    /// Are the two color space names/aliases/roles equivalent?
+    bool equivalent(string_view color_space,
+                    string_view other_color_space) const;
+
     /// Return a filename or other identifier for the config we're using.
     std::string configname() const;
 
@@ -313,8 +341,15 @@ public:
     /// OCIO support is available.
     static int OpenColorIO_version_hex();
 
+    /// Return a default ColorConfig, which is a singleton that will be
+    /// created the first time it is needed.  It will be initialized with the
+    /// OCIO environment variable, if it exists, or the OCIO built-in config
+    /// (for OCIO >= 2.2).  If neither of those is possible, it will be
+    /// initialized with a built-in minimal config.
+    static const ColorConfig& default_colorconfig();
+
 private:
-    ColorConfig(const ColorConfig&) = delete;
+    ColorConfig(const ColorConfig&)            = delete;
     ColorConfig& operator=(const ColorConfig&) = delete;
 
     class Impl;
@@ -324,7 +359,8 @@ private:
 
 
 
-/// Utility -- convert sRGB value to linear
+/// Utility -- convert sRGB value to linear transfer function, without
+/// any change in color primaries.
 ///    http://en.wikipedia.org/wiki/SRGB
 inline float
 sRGB_to_linear(float x)
@@ -344,7 +380,8 @@ sRGB_to_linear(const simd::vfloat4& x)
 }
 #endif
 
-/// Utility -- convert linear value to sRGB
+/// Utility -- convert linear value to sRGB transfer function, without
+/// any change in color primaries.
 inline float
 linear_to_sRGB(float x)
 {
@@ -354,7 +391,8 @@ linear_to_sRGB(float x)
 
 
 #ifndef __CUDA_ARCH__
-/// Utility -- convert linear value to sRGB
+/// Utility -- convert linear value to sRGB transfer function, without
+/// any change in color primaries.
 inline simd::vfloat4
 linear_to_sRGB(const simd::vfloat4& x)
 {
@@ -365,7 +403,8 @@ linear_to_sRGB(const simd::vfloat4& x)
 #endif
 
 
-/// Utility -- convert Rec709 value to linear
+/// Utility -- convert Rec709 value to linear transfer function, without
+/// any change in color primaries.
 ///    http://en.wikipedia.org/wiki/Rec._709
 inline float
 Rec709_to_linear(float x)
@@ -376,7 +415,8 @@ Rec709_to_linear(float x)
         return powf((x + 0.099f) * (1.0f / 1.099f), (1.0f / 0.45f));
 }
 
-/// Utility -- convert linear value to Rec709
+/// Utility -- convert linear value to Rec709 transfer function, without
+/// any change in color primaries.
 inline float
 linear_to_Rec709(float x)
 {
