@@ -1,5 +1,5 @@
 // Copyright Contributors to the OpenImageIO project.
-// SPDX-License-Identifier: BSD-3-Clause and Apache-2.0
+// SPDX-License-Identifier: Apache-2.0
 // https://github.com/AcademySoftwareFoundation/OpenImageIO
 
 
@@ -38,29 +38,15 @@
 #    include <malloc.h>  // for alloca
 #endif
 
-#if defined(_WIN32)
-#    ifndef WIN32_LEAN_AND_MEAN
-#        define WIN32_LEAN_AND_MEAN
-#    endif
-#    ifndef VC_EXTRALEAN
-#        define VC_EXTRALEAN
-#    endif
-#    ifndef NOMINMAX
-#        define NOMINMAX
-#    endif
-#    ifndef NOGDI
-#        define NOGDI
-#    endif
-#    include <windows.h>
-#endif
-
 #ifdef _MSC_VER
 #    include <intrin.h>
 #endif
 
-//Avoid min and max being defined
-#if defined(_WIN32)
-#define NOMINMAX
+// Avoid min and max being defined for any subsequent include of windows.h
+#ifdef _WIN32
+#    ifndef NOMINMAX
+#        define NOMINMAX
+#    endif
 #endif
 
 #include <OpenImageIO/oiioversion.h>
@@ -538,8 +524,10 @@
 // function decorators needed when compiling for CUDA devices.
 #ifdef __CUDACC__
 #    define OIIO_HOSTDEVICE __host__ __device__
+#    define OIIO_DEVICE __device__
 #else
 #    define OIIO_HOSTDEVICE
+#    define OIIO_DEVICE
 #endif
 
 
