@@ -1,5 +1,5 @@
 // Copyright Contributors to the OpenImageIO project.
-// SPDX-License-Identifier: BSD-3-Clause and Apache-2.0
+// SPDX-License-Identifier: Apache-2.0
 // https://github.com/AcademySoftwareFoundation/OpenImageIO
 
 // clang-format off
@@ -83,6 +83,7 @@ struct OIIO_UTIL_API TypeDesc {
         DOUBLE,             ///< 64-bit IEEE floating point values, (C/C++ `double`).
         STRING,             ///< Character string.
         PTR,                ///< A pointer value.
+        USTRINGHASH,        ///< The hash of a ustring
         LASTBASE
     };
 
@@ -163,10 +164,7 @@ struct OIIO_UTIL_API TypeDesc {
     TypeDesc (string_view typestring);
 
     /// Copy constructor.
-    OIIO_HOSTDEVICE constexpr TypeDesc (const TypeDesc &t) noexcept
-        : basetype(t.basetype), aggregate(t.aggregate),
-          vecsemantics(t.vecsemantics), reserved(0), arraylen(t.arraylen)
-          { }
+    OIIO_HOSTDEVICE constexpr TypeDesc (const TypeDesc &t) noexcept = default;
 
 
     /// Return the name, for printing and whatnot.  For example,
@@ -364,8 +362,13 @@ struct OIIO_UTIL_API TypeDesc {
 #endif
 };
 
-
-
+// Validate that TypeDesc can be used directly as POD in a C interface.
+static_assert(std::is_default_constructible<TypeDesc>(), "TypeDesc is not default constructable.");
+static_assert(std::is_trivially_copyable<TypeDesc>(), "TypeDesc is not trivially copyable.");
+static_assert(std::is_trivially_destructible<TypeDesc>(), "TypeDesc is not trivially destructible.");
+static_assert(std::is_trivially_move_constructible<TypeDesc>(), "TypeDesc is not move constructible.");
+static_assert(std::is_trivially_copy_constructible<TypeDesc>(), "TypeDesc is not copy constructible.");
+static_assert(std::is_trivially_move_assignable<TypeDesc>(), "TypeDesc is not move assignable.");
 
 // Static values for commonly used types. Because these are constexpr,
 // they should incur no runtime construction cost and should optimize nicely
@@ -404,6 +407,7 @@ OIIO_INLINE_CONSTEXPR TypeDesc TypeTimeCode (TypeDesc::UINT, TypeDesc::SCALAR, T
 OIIO_INLINE_CONSTEXPR TypeDesc TypeKeyCode (TypeDesc::INT, TypeDesc::SCALAR, TypeDesc::KEYCODE, 7);
 OIIO_INLINE_CONSTEXPR TypeDesc TypeRational(TypeDesc::INT, TypeDesc::VEC2, TypeDesc::RATIONAL);
 OIIO_INLINE_CONSTEXPR TypeDesc TypePointer(TypeDesc::PTR);
+OIIO_INLINE_CONSTEXPR TypeDesc TypeUstringhash(TypeDesc::USTRINGHASH);
 
 
 
@@ -485,6 +489,10 @@ template<> struct TypeDescFromC<Imath::Box3i> { static const constexpr TypeDesc
 #endif
 
 
+class ustringhash;  // forward declaration
+
+
+
 /// A template mechanism for getting C type of TypeDesc::BASETYPE.
 ///
 template<int b> struct CType {};
@@ -501,6 +509,7 @@ template<> struct CType<(int)TypeDesc::HALF> { typedef half type; };
 #endif
 template<> struct CType<(int)TypeDesc::FLOAT> { typedef float type; };
 template<> struct CType<(int)TypeDesc::DOUBLE> { typedef double type; };
+template<> struct CType<(int)TypeDesc::USTRINGHASH> { typedef ustringhash type; };
 
 
 
@@ -613,7 +622,8 @@ struct formatter<OIIO::TypeDesc> {
     }
 
     template <typename FormatContext>
-    auto format(const OIIO::TypeDesc& t, FormatContext& ctx) -> decltype(ctx.out()) const {
+    auto format(const OIIO::TypeDesc& t, FormatContext& ctx) OIIO_FMT_CUSTOM_FORMATTER_CONST
+    {
         // C++14:   auto format(const OIIO::TypeDesc& p, FormatContext& ctx) const {
         // ctx.out() is an output iterator to write to.
         return format_to(ctx.out(), "{}", t.c_str());
