@@ -3,15 +3,24 @@
 // https://github.com/AcademySoftwareFoundation/OpenImageIO
 
 
-#include <cmath>
+#include <OpenImageIO/platform.h>
+
+// Special dance to disable warnings in the included files related to
+// the deprecation of unicode conversion functions.
+OIIO_PRAGMA_WARNING_PUSH
+OIIO_CLANG_PRAGMA(clang diagnostic ignored "-Wdeprecated-declarations")
 #include <codecvt>
+#include <locale>
+OIIO_PRAGMA_WARNING_POP
+
+#include <algorithm>
+#include <cmath>
 #include <cstdarg>
 #include <cstdint>
 #include <cstdlib>
 #include <ctime>
 #include <iostream>
 #include <limits>
-#include <locale>
 #include <mutex>
 #include <numeric>
 #include <sstream>
@@ -25,16 +34,11 @@
 #include <boost/algorithm/string/find.hpp>
 
 #include <OpenImageIO/dassert.h>
-#include <OpenImageIO/platform.h>
 #include <OpenImageIO/string_view.h>
 #include <OpenImageIO/strutil.h>
 #include <OpenImageIO/thread.h>
 #include <OpenImageIO/ustring.h>
 
-#ifdef _WIN32
-#    include <shellapi.h>
-#endif
-
 
 // We use the public domain stb implementation of vsnprintf because
 // not all platforms support a locale-independent version of vsnprintf.
@@ -237,7 +241,7 @@ Strutil::vsprintf(const char* fmt, va_list ap)
 #ifdef va_copy
         va_copy(ap, apsave);
 #else
-        ap     = apsave;
+        ap = apsave;
 #endif
     }
 }
@@ -1568,8 +1572,9 @@ Strutil::stoi(string_view str, size_t* pos, int base)
         }
         if (c >= base)
             break;
-        acc       = acc * base + c;
         anydigits = true;
+        if (OIIO_LIKELY(!overflow))
+            acc = acc * base + c;
         if (OIIO_UNLIKELY(acc > maxval))
             overflow = true;
     }
@@ -1857,4 +1862,30 @@ Strutil::edit_distance(string_view a, string_view b, EditDistMetric metric)
     return levenshtein_distance(a, b);
 }
 
+
+
+/// Interpret a string as a boolean value using the following heuristic:
+///   - If the string is a valid numeric value (represents an integer or
+///     floating point value), return true if it's non-zero, false if it's
+///     zero.
+///   - If the string is one of "false", "no", or "off", or if it contains
+///     only whitespace, return false.
+///   - All other non-empty strings return true.
+/// The comparisons are case-insensitive and ignore leading and trailing
+/// whitespace.
+bool
+Strutil::eval_as_bool(string_view value)
+{
+    Strutil::trim_whitespace(value);
+    if (Strutil::string_is_int(value)) {
+        return Strutil::stoi(value) != 0;
+    } else if (Strutil::string_is_float(value)) {
+        return Strutil::stof(value) != 0.0f;
+    } else {
+        return !(value.empty() || Strutil::iequals(value, "false")
+                 || Strutil::iequals(value, "no")
+                 || Strutil::iequals(value, "off"));
+    }
+}
+
 OIIO_NAMESPACE_END
