@@ -346,13 +346,13 @@ void
 test_hash()
 {
     using namespace Strutil;
-    OIIO_CHECK_EQUAL(strhash("foo"), 6150913649986995171);
-    OIIO_CHECK_EQUAL(strhash(std::string("foo")), 6150913649986995171);
-    OIIO_CHECK_EQUAL(strhash(string_view("foo")), 6150913649986995171);
+    OIIO_CHECK_EQUAL(strhash("foo"), size_t(6150913649986995171));
+    OIIO_CHECK_EQUAL(strhash(std::string("foo")), size_t(6150913649986995171));
+    OIIO_CHECK_EQUAL(strhash(string_view("foo")), size_t(6150913649986995171));
     OIIO_CHECK_EQUAL(strhash(""), 0);  // empty string hashes to 0
     // Check longer hash and ensure that it's really constexpr
     constexpr size_t hash = Strutil::strhash("much longer string");
-    OIIO_CHECK_EQUAL(hash, 16257490369375554819ULL);
+    OIIO_CHECK_EQUAL(hash, size_t(16257490369375554819ULL));
 }
 
 
@@ -759,6 +759,38 @@ test_concat()
     std::string longstring(Strutil::repeat("01234567890", 100));
     OIIO_CHECK_EQUAL(Strutil::concat(longstring, longstring),
                      Strutil::sprintf("%s%s", longstring, longstring));
+    OIIO_CHECK_EQUAL(Strutil::concat(longstring, longstring),
+                     Strutil::fmt::format("{}{}", longstring, longstring));
+
+    Benchmarker bench;
+    bench.indent (2);
+    bench.units (Benchmarker::Unit::ns);
+    std::string foostr("foo"), barstr("bar");
+    bench ("concat literal short+short", [&](){ return DoNotOptimize(Strutil::concat("foo", "bar")); });
+    bench ("concat literal long+short", [&](){ return DoNotOptimize(Strutil::concat(longstring, "bar")); });
+    bench ("concat literal long+long", [&](){ return DoNotOptimize(Strutil::concat(longstring, longstring)); });
+    bench ("format literal short+short", [&](){ return DoNotOptimize(Strutil::fmt::format("{}{}", "foo", "bar")); });
+    bench ("format literal long+short", [&](){ return DoNotOptimize(Strutil::fmt::format("{}{}", longstring, "bar")); });
+    bench ("format literal long+long", [&](){ return DoNotOptimize(Strutil::fmt::format("{}{}", longstring, longstring)); });
+    bench ("sprintf literal short+short", [&](){ return DoNotOptimize(Strutil::sprintf("%s%s", "foo", "bar")); });
+    bench ("sprintf literal long+short", [&](){ return DoNotOptimize(Strutil::sprintf("%s%s", longstring, "bar")); });
+    bench ("sprintf literal long+long", [&](){ return DoNotOptimize(Strutil::sprintf("%s%s", longstring, longstring)); });
+
+    bench ("concat str short+short", [&](){ return DoNotOptimize(Strutil::concat(foostr, barstr)); });
+    bench ("concat str long+short", [&](){ return DoNotOptimize(Strutil::concat(longstring, barstr)); });
+    bench ("concat str long+long", [&](){ return DoNotOptimize(Strutil::concat(longstring, longstring)); });
+    bench ("format str short+short", [&](){ return DoNotOptimize(Strutil::fmt::format("{}{}", foostr, barstr)); });
+    bench ("format str long+short", [&](){ return DoNotOptimize(Strutil::fmt::format("{}{}", longstring, barstr)); });
+    bench ("format str long+long", [&](){ return DoNotOptimize(Strutil::fmt::format("{}{}", longstring, longstring)); });
+    bench ("sprintf str short+short", [&](){ return DoNotOptimize(Strutil::sprintf("%s%s", foostr, barstr)); });
+    bench ("sprintf str long+short", [&](){ return DoNotOptimize(Strutil::sprintf("%s%s", longstring, barstr)); });
+    bench ("sprintf str long+long", [&](){ return DoNotOptimize(Strutil::sprintf("%s%s", longstring, longstring)); });
+    bench ("std::string + literal short+short", [&](){ return DoNotOptimize(std::string("foo") + std::string("bar")); });
+    bench ("std::string + literal long+short", [&](){ return DoNotOptimize(longstring + std::string("bar")); });
+    bench ("std::string + literal long+long", [&](){ return DoNotOptimize(longstring + longstring); });
+    bench ("std::string + str short+short", [&](){ return DoNotOptimize(foostr + barstr); });
+    bench ("std::string + str long+short", [&](){ return DoNotOptimize(longstring + barstr); });
+    bench ("std::string + str long+long", [&](){ return DoNotOptimize(longstring + longstring); });
 }
 
 
@@ -869,6 +901,7 @@ test_numeric_conversion()
     OIIO_CHECK_EQUAL(Strutil::stoi("-12345678901234567890"),
                      std::numeric_limits<int>::min());
     OIIO_CHECK_EQUAL(Strutil::stoi("0x100", nullptr, 16), 256);  // hex
+    OIIO_CHECK_EQUAL(Strutil::stoi("25555555555555555551"), 2147483647);
 
     OIIO_CHECK_EQUAL(Strutil::stoui("hi"), 0);
     OIIO_CHECK_EQUAL(Strutil::stoui("  "), 0);
@@ -1466,6 +1499,16 @@ void test_parse ()
     OIIO_CHECK_EQUAL (ss, ""); OIIO_CHECK_EQUAL (s, "");
     s = "(blah"; ss = parse_nested (s);
     OIIO_CHECK_EQUAL (ss, ""); OIIO_CHECK_EQUAL (s, "(blah");
+
+    OIIO_CHECK_EQUAL(string_is_identifier("valid"), true);
+    OIIO_CHECK_EQUAL(string_is_identifier("_underscore"), true);
+    OIIO_CHECK_EQUAL(string_is_identifier("with123numbers"), true);
+    OIIO_CHECK_EQUAL(string_is_identifier("123invalidStart"), false);
+    OIIO_CHECK_EQUAL(string_is_identifier("invalid-char"), false);
+    OIIO_CHECK_EQUAL(string_is_identifier(""), false);
+    OIIO_CHECK_EQUAL(string_is_identifier("a"), true);
+    OIIO_CHECK_EQUAL(string_is_identifier("_"), true);
+    OIIO_CHECK_EQUAL(string_is_identifier("1"), false);
 }
 
 
@@ -1641,6 +1684,53 @@ test_base64_encode()
 
 
 
+void
+test_eval_as_bool()
+{
+    using namespace Strutil;
+    print("testing eval_as_bool()\n");
+
+    // Test cases for integer values
+    OIIO_CHECK_EQUAL(eval_as_bool("0"), false);
+    OIIO_CHECK_EQUAL(eval_as_bool("1"), true);
+    OIIO_CHECK_EQUAL(eval_as_bool("-1"), true);
+    OIIO_CHECK_EQUAL(eval_as_bool("10"), true);
+    OIIO_CHECK_EQUAL(eval_as_bool("-10"), true);
+
+    // Test cases for floating-point values
+    OIIO_CHECK_EQUAL(eval_as_bool("0.0"), false);
+    OIIO_CHECK_EQUAL(eval_as_bool("1.0"), true);
+    OIIO_CHECK_EQUAL(eval_as_bool("-1.0"), true);
+    OIIO_CHECK_EQUAL(eval_as_bool("10.5"), true);
+    OIIO_CHECK_EQUAL(eval_as_bool("-10.5"), true);
+
+    // Test cases for string values
+    OIIO_CHECK_EQUAL(eval_as_bool(""), false);
+    OIIO_CHECK_EQUAL(eval_as_bool("false"), false);
+    OIIO_CHECK_EQUAL(eval_as_bool("FALSE"), false);
+    OIIO_CHECK_EQUAL(eval_as_bool("no"), false);
+    OIIO_CHECK_EQUAL(eval_as_bool("NO"), false);
+    OIIO_CHECK_EQUAL(eval_as_bool("off"), false);
+    OIIO_CHECK_EQUAL(eval_as_bool("OFF"), false);
+
+    OIIO_CHECK_EQUAL(eval_as_bool("true"), true);
+    OIIO_CHECK_EQUAL(eval_as_bool("TRUE"), true);
+    OIIO_CHECK_EQUAL(eval_as_bool("yes"), true);
+    OIIO_CHECK_EQUAL(eval_as_bool("YES"), true);
+    OIIO_CHECK_EQUAL(eval_as_bool("on"), true);
+    OIIO_CHECK_EQUAL(eval_as_bool("ON"), true);
+    OIIO_CHECK_EQUAL(eval_as_bool("OpenImageIO"), true);
+
+    // Test whitespace, case insensitivity, other tricky cases
+    OIIO_CHECK_EQUAL(eval_as_bool("   "), false);
+    OIIO_CHECK_EQUAL(eval_as_bool("\t \n"), false);
+    OIIO_CHECK_EQUAL(eval_as_bool(" faLsE"), false);
+    OIIO_CHECK_EQUAL(eval_as_bool("\tOFf"), false);
+    OIIO_CHECK_EQUAL(eval_as_bool("off OpenImageIO"), true);
+}
+
+
+
 int
 main(int /*argc*/, char* /*argv*/[])
 {
@@ -1676,6 +1766,7 @@ main(int /*argc*/, char* /*argv*/[])
     test_datetime();
     test_edit_distance();
     test_base64_encode();
+    test_eval_as_bool();
 
     Strutil::debug("debug message\n");
 
