@@ -14,6 +14,7 @@
 #include <map>
 #include <memory>
 #include <vector>
+#include <zlib.h>
 
 #include <OpenImageIO/fmath.h>
 #include <OpenImageIO/imagebuf.h>
@@ -38,6 +39,7 @@ public:
         return (feature == "exif" || feature == "iptc" || feature == "thumbnail"
                 || feature == "ioproxy");
     }
+    bool valid_file(Filesystem::IOProxy* ioproxy) const override;
     bool open(const std::string& name, ImageSpec& newspec) override;
     bool open(const std::string& name, ImageSpec& newspec,
               const ImageSpec& config) override;
@@ -127,6 +129,15 @@ private:
         uint64_t data_length;
         int64_t data_pos;
         uint16_t compression;
+
+        uint32_t width;
+        uint32_t height;
+
+        // This data is only relevant for the compression
+        // codecs zip and zipprediction as we need to preallocate
+        // the memory this vector is already decompressed and byteswapped
+        std::vector<char> decompressed_data;
+
         std::vector<uint32_t> rle_lengths;
         std::vector<int64_t> row_pos;
     };
@@ -223,6 +234,7 @@ private:
     bool load_header();
     bool read_header();
     bool validate_header();
+    static bool validate_signature(const char signature[4]);
 
     //Color Mode Data
     bool load_color_data();
@@ -274,6 +286,9 @@ private:
     //Global Additional Layer Info
     bool load_global_additional();
 
+    //Layers for 16- and 32-bit documents
+    bool load_layers_16_32(uint64_t length);
+
     //Image Data Section
     bool load_image_data();
 
@@ -283,8 +298,7 @@ private:
     void fill_channel_names(ImageSpec& spec, bool transparency);
 
     //Read a row of channel data
-    bool read_channel_row(const ChannelInfo& channel_info, uint32_t row,
-                          char* data);
+    bool read_channel_row(ChannelInfo& channel_info, uint32_t row, char* data);
 
     // Interleave channels (RRRGGGBBB -> RGBRGBRGB) while copying from
     // channel_buffers[0..nchans-1] to dst.
@@ -412,8 +426,17 @@ private:
 
     int read_pascal_string(std::string& s, uint16_t mod_padding);
 
+    // Swap a planar bytespan representing the bytes of a float vector to its
+    // interleaved byte order. This is per scanline
+    void float_planar_to_interleaved(span<char> data, size_t width,
+                                     size_t height);
+
+    // All the compression modes known to photoshop
     bool decompress_packbits(const char* src, char* dst, uint32_t packed_length,
                              uint32_t unpacked_length);
+    bool decompress_zip(span<char> src, span<char> dest);
+    bool decompress_zip_prediction(span<char> src, span<char> dest,
+                                   const uint32_t width, const uint32_t height);
 
     // These are AdditionalInfo entries that, for PSBs, have an 8-byte length
     static const char* additional_info_psb[];
@@ -522,9 +545,32 @@ OIIO_PLUGIN_EXPORTS_END
 
 
 
+/// Byteswap a span of values.  This should work for any of short, unsigned
+/// short, int, unsigned int, float, long long, pointers. It's just calling
+/// byteswap on each element.
+template<typename T>
+inline OIIO_HOSTDEVICE void
+byteswap_span(span<T> vals)
+{
+    swap_endian(vals.data(), int(vals.size()));
+}
+
+
 PSDInput::PSDInput() { init(); }
 
 
+bool
+PSDInput::valid_file(Filesystem::IOProxy* ioproxy) const
+{
+    if (!ioproxy || ioproxy->mode() != Filesystem::IOProxy::Mode::Read)
+        return false;
+
+    char signature[4] {};
+    const size_t numRead = ioproxy->pread(signature, sizeof(signature), 0);
+    return numRead == sizeof(signature) && validate_signature(signature);
+}
+
+
 
 bool
 PSDInput::open(const std::string& name, ImageSpec& newspec)
@@ -901,10 +947,18 @@ PSDInput::read_header()
 
 
 
+bool
+PSDInput::validate_signature(const char signature[4])
+{
+    return std::memcmp(signature, "8BPS", 4) == 0;
+}
+
+
+
 bool
 PSDInput::validate_header()
 {
-    if (std::memcmp(m_header.signature, "8BPS", 4) != 0) {
+    if (!validate_signature(m_header.signature)) {
         errorfmt("[Header] invalid signature");
         return false;
     }
@@ -1085,7 +1139,8 @@ PSDInput::handle_resources(ImageResourceMap& resources)
 
 
 
-bool PSDInput::load_resource_1005(uint32_t /*length*/)
+bool
+PSDInput::load_resource_1005(uint32_t /*length*/)
 {
     ResolutionInfo resinfo;
     // Fixed 16.16
@@ -1147,7 +1202,8 @@ PSDInput::load_resource_1006(uint32_t length)
 
 
 
-bool PSDInput::load_resource_1010(uint32_t /*length*/)
+bool
+PSDInput::load_resource_1010(uint32_t /*length*/)
 {
     int8_t color_id = 0;
     int32_t color   = 0;
@@ -1197,7 +1253,8 @@ PSDInput::load_resource_1039(uint32_t length)
 
 
 
-bool PSDInput::load_resource_1047(uint32_t /*length*/)
+bool
+PSDInput::load_resource_1047(uint32_t /*length*/)
 {
     if (!read_bige<int16_t>(m_transparency_index))
         return false;
@@ -1255,7 +1312,8 @@ PSDInput::load_resource_1060(uint32_t length)
 
 
 
-bool PSDInput::load_resource_1064(uint32_t /*length*/)
+bool
+PSDInput::load_resource_1064(uint32_t /*length*/)
 {
     uint32_t version;
     if (!read_bige<uint32_t>(version))
@@ -1393,6 +1451,11 @@ PSDInput::load_layers()
     if (!ok)
         return false;
 
+    // There is 3 cases where this could be empty:
+    // - 16-bit files store this data in the Lr16 tagged block
+    // - 32-bit files store this data in the Lr32 tagged block
+    // - Single layer files may optimize this section away and instead
+    //   store the data on the merged image data section
     if (!layer_info.length)
         return true;
 
@@ -1556,17 +1619,20 @@ PSDInput::load_layer_channel(Layer& layer, ChannelInfo& channel_info)
     uint32_t width, height;
     if (channel_info.channel_id == ChannelID_LayerMask) {
         width  = (uint32_t)std::abs((int)layer.mask_data.right
-                                   - (int)layer.mask_data.left);
+                                    - (int)layer.mask_data.left);
         height = (uint32_t)std::abs((int)layer.mask_data.bottom
                                     - (int)layer.mask_data.top);
     } else {
         width  = layer.width;
         height = layer.height;
     }
+    channel_info.width  = width;
+    channel_info.height = height;
 
     channel_info.data_pos = iotell();
     channel_info.row_pos.resize(height);
     channel_info.row_length = (width * m_header.depth + 7) / 8;
+
     switch (channel_info.compression) {
     case Compression_Raw:
         if (height) {
@@ -1576,6 +1642,9 @@ PSDInput::load_layer_channel(Layer& layer, ChannelInfo& channel_info)
                                           + channel_info.row_length;
         }
         channel_info.data_length = channel_info.row_length * height;
+
+        if (!ioseek(channel_info.data_length, SEEK_CUR))
+            return false;
         break;
     case Compression_RLE:
         // RLE lengths are stored before the channel data
@@ -1593,19 +1662,52 @@ PSDInput::load_layer_channel(Layer& layer, ChannelInfo& channel_info)
                 channel_info.row_pos[i] = channel_info.row_pos[i - 1]
                                           + channel_info.rle_lengths[i - 1];
         }
+
+        if (!ioseek(channel_info.data_length, SEEK_CUR))
+            return false;
         break;
-    // These two aren't currently supported. They would likely
-    // require large changes in the code as they probably don't
-    // support random access like the other modes. I doubt these are
-    // used much and I haven't found any test images.
-    case Compression_ZIP:
-    case Compression_ZIP_Predict:
+    case Compression_ZIP: {
+        // We subtract the compression marker from the data length
+        channel_info.data_length -= 2;
+
+        // Unlike with raw and rle compression we cannot access each scanline
+        // randomly so we parse the data up-front and store it
+        std::vector<char> compressed_data(channel_info.data_length);
+        channel_info.decompressed_data = std::vector<char>(
+            width * height * (m_header.depth / 8));
+
+        if (!ioseek(channel_info.data_pos))
+            return false;
+        if (!ioread(compressed_data.data(), channel_info.data_length))
+            return false;
+
+        decompress_zip(compressed_data, channel_info.decompressed_data);
+    } break;
+    case Compression_ZIP_Predict: {
+        // We subtract the compression marker from the data length
+        channel_info.data_length -= 2;
+
+        // Unlike with raw and rle compression we cannot access each scanline
+        // randomly so we parse the data up-front and store it
+        std::vector<char> compressed_data(channel_info.data_length);
+        channel_info.decompressed_data = std::vector<char>(
+            width * height * (m_header.depth / 8));
+
+        if (!ioseek(channel_info.data_pos))
+            return false;
+        if (!ioread(compressed_data.data(), channel_info.data_length))
+            return false;
+
+        decompress_zip_prediction(compressed_data,
+                                  channel_info.decompressed_data, width,
+                                  height);
+    } break;
     default:
         errorfmt("[Layer Channel] unsupported compression {}",
                  channel_info.compression);
         return false;
     }
-    return ioseek(channel_info.data_length, SEEK_CUR);
+    return true;
 }
 
 
@@ -1637,7 +1739,10 @@ PSDInput::load_global_mask_info()
     uint32_t length;
 
     // This section should be at least 17 bytes, but some files lack
-    // global mask info and additional layer info, not covered in the spec
+    // global mask info and additional layer info, not covered in the spec.
+    // More modern photoshop files appear to omit this section entirely.
+    // We leave this code here though in case we want to deal with older photoshop files
+    // although it is not currently used anywhere else
     if (remaining < 17) {
         return ioseek(m_layer_mask_info.end);
     }
@@ -1700,11 +1805,20 @@ PSDInput::load_global_additional()
         // Long story short these are aligned to 4 bytes but that is not
         // included in the stored length and the specs do not mention it.
 
-        // round up to multiple of 4
-        length = (length + 3) & ~3;
-        remaining -= length;
-        // skip it for now
-        ok &= ioseek(length, SEEK_CUR);
+        // Load 16 and 32-bit layer data
+        if (std::memcmp(key, "Lr16", 4) == 0
+            || std::memcmp(key, "Lr32", 4) == 0) {
+            uint64_t begin_offset = iotell();
+            ok &= load_layers_16_32(length);
+            uint64_t size = iotell() - begin_offset;
+            remaining -= size;
+        } else {
+            // round up to multiple of 4
+            length = (length + 3) & ~3;
+            remaining -= length;
+            // skip it for now
+            ok &= ioseek(length, SEEK_CUR);
+        }
     }
     // finished with the layer and mask information section, seek to the end
     ok &= ioseek(m_layer_mask_info.end);
@@ -1712,6 +1826,53 @@ PSDInput::load_global_additional()
 }
 
 
+bool
+PSDInput::load_layers_16_32(uint64_t length)
+{
+    // Notice that, bar the reading of the length marker, reading this section is identical to
+    // the normal layer info section
+    bool ok = true;
+
+    if (length == 0)
+        return false;
+
+    LayerMaskInfo::LayerInfo& layer_info = m_layer_mask_info.layer_info;
+    // The layer info length must have been 0 in the actual layer info section
+    OIIO_ASSERT(layer_info.length == 0);
+    layer_info.length = length;
+
+    uint64_t begin = iotell();
+
+    // We read the layer info as we would usually since the section is exactly the same
+    ok &= read_bige<int16_t>(layer_info.layer_count);
+    if (layer_info.layer_count < 0) {
+        m_image_data.transparency = true;
+        layer_info.layer_count    = -layer_info.layer_count;
+    }
+    m_layers.resize(layer_info.layer_count);
+    for (int16_t layer_nbr = 0; layer_nbr < layer_info.layer_count;
+         ++layer_nbr) {
+        Layer& layer = m_layers[layer_nbr];
+        if (!load_layer(layer))
+            return false;
+    }
+    for (int16_t layer_nbr = 0; layer_nbr < layer_info.layer_count;
+         ++layer_nbr) {
+        Layer& layer = m_layers[layer_nbr];
+        if (!load_layer_channels(layer))
+            return false;
+    }
+
+    // This section, like the other tagged blocks are padded to 4 bytes
+    uint64_t length_read = iotell() - begin;
+    int64_t remaining    = (((length_read + 3) / 4) * 4) - length_read;
+    OIIO_ASSERT(remaining >= 0);
+    OIIO_ASSERT(remaining < 4);
+    ioseek(remaining, SEEK_CUR);
+
+    return ok;
+}
+
 
 bool
 PSDInput::load_image_data()
@@ -1731,6 +1892,8 @@ PSDInput::load_image_data()
     // setup some generic properties and read any RLE lengths
     // Image Data Section has RLE lengths for all channels stored first
     for (ChannelInfo& channel_info : m_image_data.channel_info) {
+        channel_info.width       = m_header.width;
+        channel_info.height      = m_header.height;
         channel_info.compression = compression;
         channel_info.channel_id  = id++;
         channel_info.data_length = row_length * m_header.height;
@@ -1866,8 +2029,7 @@ PSDInput::fill_channel_names(ImageSpec& spec, bool transparency)
 
 
 bool
-PSDInput::read_channel_row(const ChannelInfo& channel_info, uint32_t row,
-                           char* data)
+PSDInput::read_channel_row(ChannelInfo& channel_info, uint32_t row, char* data)
 {
     if (row >= channel_info.row_pos.size()) {
         errorfmt("Reading channel row out of range ({}, should be < {})", row,
@@ -1875,14 +2037,23 @@ PSDInput::read_channel_row(const ChannelInfo& channel_info, uint32_t row,
         return false;
     }
 
-    if (!ioseek(channel_info.row_pos[row]))
-        return false;
     switch (channel_info.compression) {
     case Compression_Raw:
+        if (!ioseek(channel_info.row_pos[row]))
+            return false;
         if (!ioread(data, channel_info.row_length))
             return false;
+
+        if (!bigendian()) {
+            switch (m_header.depth) {
+            case 16: swap_endian((uint16_t*)data, channel_info.width); break;
+            case 32: swap_endian((uint32_t*)data, channel_info.width); break;
+            }
+        }
         break;
     case Compression_RLE: {
+        if (!ioseek(channel_info.row_pos[row]))
+            return false;
         uint32_t rle_length = channel_info.rle_lengths[row];
         char* rle_buffer;
         OIIO_ALLOCATE_STACK_OR_HEAP(rle_buffer, char, rle_length);
@@ -1890,24 +2061,29 @@ PSDInput::read_channel_row(const ChannelInfo& channel_info, uint32_t row,
             || !decompress_packbits(rle_buffer, data, rle_length,
                                     channel_info.row_length))
             return false;
-        break;
-    }
+    } break;
+    case Compression_ZIP: {
+        OIIO_ASSERT(channel_info.decompressed_data.size()
+                    == static_cast<uint64_t>(channel_info.width)
+                           * channel_info.height * (m_header.depth / 8));
+        // We simply copy over the row into destination
+        uint64_t row_index = static_cast<uint64_t>(row) * channel_info.width
+                             * (m_header.depth / 8);
+        std::memcpy(data, channel_info.decompressed_data.data() + row_index,
+                    channel_info.row_length);
+    } break;
+    case Compression_ZIP_Predict: {
+        OIIO_ASSERT(channel_info.decompressed_data.size()
+                    == static_cast<uint64_t>(channel_info.width)
+                           * channel_info.height * (m_header.depth / 8));
+        // We simply copy over the row into destination
+        uint64_t row_index = static_cast<uint64_t>(row) * channel_info.width
+                             * (m_header.depth / 8);
+        std::memcpy(data, channel_info.decompressed_data.data() + row_index,
+                    channel_info.row_length);
+    } break;
     }
 
-    if (!bigendian()) {
-        switch (m_header.depth) {
-        case 16: swap_endian((uint16_t*)data, m_spec.width); break;
-        case 32:
-            swap_endian((uint32_t*)data, m_spec.width);
-            // if (row == 131)
-            //     printf ("%x %x %x %x\n",
-            //             ((uint32_t*)data)[0], ((uint32_t*)data)[1],
-            //             ((uint32_t*)data)[2], ((uint32_t*)data)[3]);
-            // convert_type<float,uint32_t> ((float *)&data[0],
-            //                               (uint32_t*)&data[1], m_spec.width);
-            break;
-        }
-    }
     return true;
 }
 
@@ -2024,6 +2200,31 @@ PSDInput::read_pascal_string(std::string& s, uint16_t mod_padding)
 
 
 
+void
+PSDInput::float_planar_to_interleaved(span<char> data, size_t width,
+                                      size_t height)
+{
+    std::vector<char> buffer(data.size());
+
+    // Shuffle from planar 1111... 2222... 3333... 4444... byte order to 1234 1234 1234 1234...
+    for (uint64_t y = 0; y < height; ++y) {
+        for (uint64_t x = 0; x < width; ++x) {
+            uint64_t rowIndex = y * width * sizeof(float);
+
+            buffer[rowIndex + x * sizeof(float) + 0] = data[rowIndex + x];
+            buffer[rowIndex + x * sizeof(float) + 1]
+                = data[rowIndex + width + x];
+            buffer[rowIndex + x * sizeof(float) + 2]
+                = data[rowIndex + width * 2 + x];
+            buffer[rowIndex + x * sizeof(float) + 3]
+                = data[rowIndex + width * 3 + x];
+        }
+    }
+    std::memcpy(data.data(), buffer.data(), buffer.size());
+}
+
+
+
 bool
 PSDInput::decompress_packbits(const char* src, char* dst,
                               uint32_t packed_length, uint32_t unpacked_length)
@@ -2072,11 +2273,124 @@ PSDInput::decompress_packbits(const char* src, char* dst,
             dst += length;
         }
     }
+
+    if (!bigendian()) {
+        switch (m_header.depth) {
+        case 16: swap_endian((uint16_t*)dst, m_spec.width); break;
+        case 32: swap_endian((uint32_t*)dst, m_spec.width); break;
+        }
+    }
+
     return true;
 }
 
 
 
+bool
+PSDInput::decompress_zip(span<char> src, span<char> dest)
+{
+    z_stream stream {};
+    stream.zfree     = Z_NULL;
+    stream.opaque    = Z_NULL;
+    stream.avail_in  = src.size();
+    stream.next_in   = (Bytef*)src.data();
+    stream.avail_out = dest.size();
+    stream.next_out  = (Bytef*)dest.data();
+
+    if (inflateInit(&stream) != Z_OK) {
+        errorfmt(
+            "zip compression inflate init failed with: src_size={}, dst_size={}",
+            src.size(), dest.size());
+        return false;
+    }
+
+    if (inflate(&stream, Z_FINISH) != Z_STREAM_END) {
+        errorfmt(
+            "unable to decode zip compressed data: src_size={}, dst_size={}",
+            src.size(), dest.size());
+        return false;
+    }
+
+    if (inflateEnd(&stream) != Z_OK) {
+        errorfmt(
+            "zip compression inflate cleanup failed with: src_size={}, dst_size={}",
+            src.size(), dest.size());
+        return false;
+    }
+
+    return true;
+}
+
+
+
+bool
+PSDInput::decompress_zip_prediction(span<char> src, span<char> dest,
+                                    const uint32_t width, const uint32_t height)
+{
+    OIIO_ASSERT(width * height * (m_header.depth / 8) == dest.size());
+    bool ok = true;
+    // Decompress into dest first and then apply the prediction decoding
+    // on dest
+    ok &= decompress_zip(src, dest);
+
+    switch (m_header.depth) {
+    case 8:
+        for (uint64_t y = 0; y < height; ++y) {
+            // Index x beginning at one since we look behind to calculate
+            // the offset
+            for (uint64_t x = 1; x < width; ++x) {
+                dest[y * width + x] += dest[y * width + x - 1];
+            };
+        };
+        break;
+    case 16: {
+        // 16-bit data requires endian swapping at this point already for the
+        // prediction decoding to work correctly
+        span<uint16_t> destView(reinterpret_cast<uint16_t*>(dest.data()),
+                                dest.size() / 2);
+        if (!bigendian())
+            byteswap_span(destView);
+
+        for (uint64_t y = 0; y < height; ++y) {
+            // Index x beginning at one since we look behind to calculate
+            // the offset
+            for (uint64_t x = 1; x < width; ++x) {
+                destView[y * width + x] += destView[y * width + x - 1];
+            };
+        };
+    } break;
+    case 32: {
+        // 32-bit files actually have the float bytes stored in planar fashion on disk
+        // which are then prediction encoded. Thus we first decode the bytes itself
+        uint64_t index = 0;
+        for (uint64_t y = 0; y < height; ++y) {
+            ++index;
+            for (uint64_t x = 1; x < (width * sizeof(float)); ++x) {
+                uint8_t value = dest[index] + dest[index - 1];
+                dest[index]   = value;
+                ++index;
+            }
+        }
+
+        // We now shuffle the byte order back into place from planar to interleaved
+        float_planar_to_interleaved(dest, width, height);
+
+        // Finally we byteswap if necessary
+        if (!bigendian())
+            byteswap_span(
+                span<uint32_t>(reinterpret_cast<uint32_t*>(dest.data()),
+                               dest.size() / 4));
+    } break;
+    default:
+        errorfmt("Unknown bitdepth: {} encountered", m_header.depth);
+        return false;
+    }
+
+    return ok;
+};
+
+
+
 bool
 PSDInput::is_additional_info_psb(const char* key)
 {
