@@ -9,6 +9,6 @@
 redirect = " >> out.txt 2>&1 "
 failureok = 1
 
-command += rw_command (OIIO_TESTSUITE_IMAGEDIR + "/png/broken",
+command += rw_command (OIIO_TESTSUITE_IMAGEDIR + "/broken",
                        "invalid_gray_alpha_sbit.png",
                        printinfo=False)
